"{ Package: 'stx:goodies/petitparser/compiler/tests' }"

"{ NameSpace: Smalltalk }"

TestCase subclass:#PEGFsaDeterminizationTest
	instanceVariableNames:'fsa a b c result d interpreter e'
	classVariableNames:''
	poolDictionaries:''
	category:'PetitCompiler-Tests-FSA'
!

!PEGFsaDeterminizationTest methodsFor:'as yet unclassified'!

assert: anFsa fail: input
    | stream |
    stream := input asPetitStream.

    result := interpreter interpret: anFsa on: stream.

    self assert: result isEmpty.
    ^ result
!

assert: anFsa parse: input retval: name
    ^ self assert: anFsa parse: input retval: name end: input size
!

assert: anFsa parse: input retval: name end: end
    | stream |
    stream := input asPetitStream.

    result := interpreter interpret: anFsa on: stream.

    self assert: result isEmpty not.
    self assert: ((result at: name) = end) description: 'wrong position'.
    
    ^ result
!

assertFail: name
    self assert: (result includesKey: name) not
!

assertPass: name
    self assert: (result includesKey: name)
!

setUp
    a := PEGFsaState new name: #a; retval: #a; yourself.
    b := PEGFsaState new name: #b; retval: #b; yourself.
    c := PEGFsaState new name: #c; retval: #c; yourself.
    d := PEGFsaState new name: #d; retval: #d; yourself.
    e := PEGFsaState new name: #e; retval: #e; yourself.

    fsa := PEGFsa new.

    interpreter := PEGFsaInterpret new
        yourself.
!

testAAplusA
    fsa addState: a.
    fsa addState: b.
    fsa addState: c.
    fsa addState: d.
    fsa addState: e.
    fsa startState: a.
    fsa finalState: e.
    
    fsa addTransitionFrom: a to: b on: $a.
    fsa addTransitionFrom: b to: c on: $a.
    fsa addTransitionFrom: c to: a.	
    fsa addTransitionFrom: c to: d priority: -1. 
    fsa addTransitionFrom: d to: e on: $a.
    
    c priority: 0.
    
    fsa determinize.

"	self assert: fsa states size = 3."
    self assert: fsa isDeterministic.
    self assert: fsa isWithoutEpsilons.	
    
    self assert: fsa fail: 'a'.
    self assert: fsa fail: 'aa'.
    self assert: fsa fail: 'aaaa'.

    self assert: fsa parse: 'aaa' retval: #e.
    self assert: fsa parse: 'aaaaa' retval: #e.
    self assert: fsa parse: 'aaaaaaa' retval: #e.
!

testAB
    fsa addState: a.
    fsa addState: b.
    fsa addState: c.
    fsa addState: d.
    fsa startState: a.
    fsa finalState: d.
    
    fsa addTransitionFrom: a to: b on: $a.
    fsa addTransitionFrom: c to: d on: $b.	
    fsa addTransitionFrom: b to: c priority: -1. 
    
    fsa determinize.

    self assert: fsa states size = 3.
    self assert: fsa isDeterministic.
    self assert: fsa isWithoutEpsilons.	
    
    self assert: fsa parse: 'ab' retval: #d.
    self assert: fsa parse: 'abc' retval: #d end: 2.
    
    self assert: fsa fail: 'ac'.
!

testAPlusA
    fsa addState: a.
    fsa addState: b.
    fsa addState: c.
    fsa addState: d.
    fsa startState: a.
    fsa finalState: d.
    
    fsa addTransitionFrom: a to: b on: $a.
    fsa addTransitionFrom: b to: a.	
    fsa addTransitionFrom: b to: c priority: -1. 
    fsa addTransitionFrom: c to: d on: $a.
    
    b priority: 0.
    
    fsa determinize.

"	self assert: fsa states size = 2."
    self assert: fsa isDeterministic.
    self assert: fsa isWithoutEpsilons.	
    
    self assert: fsa fail: 'a'.
    self assert: fsa fail: 'aa'.
    self assert: fsa fail: 'b'.
!

testAPlusB
    fsa addState: a.
    fsa addState: b.
    fsa addState: c.
    fsa addState: d.
    fsa startState: a.
    fsa finalState: d.
    
    fsa addTransitionFrom: a to: b on: $a.
    fsa addTransitionFrom: b to: a.	
    fsa addTransitionFrom: b to: c priority: -1. 
    fsa addTransitionFrom: c to: d on: $b.
    
    fsa determinize.

    self assert: fsa states size = 3.
    self assert: fsa isDeterministic.
    self assert: fsa isWithoutEpsilons.	
    
    self assert: fsa parse: 'ab' retval: #d.
    self assert: fsa parse: 'aaaab' retval: #d.
    self assert: fsa parse: 'aaaabc' retval: #d end: 5.
    
    self assert: fsa fail: 'b'.
!

testAorA
    fsa addState: a.
    fsa addState: b.
    fsa addState: c.
    fsa addState: d.
    fsa addState: e.
    fsa startState: a.
    fsa finalState: c.
    fsa finalState: e.
    
    fsa addTransitionFrom: a to: b.
    fsa addTransitionFrom: a to: d.	
    fsa addTransitionFrom: b to: c on: $a. 
    fsa addTransitionFrom: d to: e on: $a. 
    
    c priority: 0.
    e priority: 0.
    
    fsa determinize.

    self assert: fsa states size = 2.
    self assert: fsa isDeterministic.
    self assert: fsa isWithoutEpsilons.	
    
    self assert: fsa parse: 'a' retval: #c.
    self assert: fsa parse: 'a' retval: #e.
    self assert: (a transitions allSatisfy: [:t | t priority = 0]).
    
    self assert: fsa fail: 'b'.
!

testApriorityOrA
    fsa addState: a.
    fsa addState: b.
    fsa addState: c.
    fsa addState: d.
    fsa addState: e.
    fsa startState: a.
    fsa finalState: c.
    fsa finalState: e.
    
    c priority: 0.
    e priority: 0.
    
    fsa addTransitionFrom: a to: b priority: -1.
    fsa addTransitionFrom: a to: d.	
    fsa addTransitionFrom: b to: c on: $a. 
    fsa addTransitionFrom: d to: e on: $a. 
    
    fsa determinize.

    self assert: fsa states size = 2.
    self assert: fsa isDeterministic.
    self assert: fsa isWithoutEpsilons.	
    
    self assert: fsa parse: 'a' retval: #e.
    self assertFail: #c.
    
    self assert: fsa fail: 'b'.
!

testApriorityOrA2
    fsa addState: a.
    fsa addState: b.
    fsa addState: c.
    fsa addState: d.
    fsa addState: e.
    fsa startState: a.
    fsa finalState: c.
    fsa finalState: e.
    
    c priority: 0.
    e priority: 0.
    
    fsa addTransitionFrom: a to: b.
    fsa addTransitionFrom: a to: d priority: -1.	
    fsa addTransitionFrom: b to: c on: $a. 
    fsa addTransitionFrom: d to: e on: $a. 
    
    fsa determinize.

    self assert: fsa states size = 2.
    self assert: fsa isDeterministic.
    self assert: fsa isWithoutEpsilons.	
    
    self assert: fsa parse: 'a' retval: #c.
    self assertFail: #e.
    
    self assert: fsa fail: 'b'.
! !

