"{ Package: 'stx:goodies/petitparser/compiler/tests' }"

"{ NameSpace: Smalltalk }"

TestCase subclass:#PPCScannerCodeGeneratorTest
	instanceVariableNames:'fsa a b c d e codeGenerator scanner result'
	classVariableNames:''
	poolDictionaries:''
	category:'PetitCompiler-Tests-Scanner'
!

!PPCScannerCodeGeneratorTest methodsFor:'as yet unclassified'!

fail: stream rule: rule 
    scanner initialize.
    scanner stream: stream asPetitStream. 
    result := scanner perform: rule.
        
    self assert: result isEmpty
!

parse: stream token: token rule: rule
    self parse: stream token: token rule: rule position: stream size.
!

parse: stream token: token rule: rule position: position
    scanner initialize.
    scanner stream: stream asPetitStream. 
    result := scanner perform: rule.
        
    self assert: (result at: token) = position.
!

setUp
    a := PEGFsaState new name: #a; retval: #a; yourself.
    b := PEGFsaState new name: #b; retval: #b; yourself.
    c := PEGFsaState new name: #c; retval: #c; yourself.
    d := PEGFsaState new name: #d; retval: #d; yourself.
    e := PEGFsaState new name: #e; retval: #e; yourself.

    fsa := PEGFsa new.
    
    codeGenerator := PPCScannerCodeGenerator new.
!

testA
    fsa addState: a.
    fsa addState: b.
    
    fsa startState: a.
    fsa finalState: b.
    
    fsa addTransitionFrom: a to: b on: $a.
    fsa name: #nextTokenA.
    b retval: #a.
    
    scanner := (codeGenerator generate: fsa).
    
    self parse: 'aaa' token: #a rule: #nextTokenA position: 1.
    self fail: 'b' rule: #nextTokenA.	
!

testAAstarA
    fsa addState: a.
    fsa addState: b.
    fsa addState: c.
    fsa startState: a.
    fsa finalState: b.
    
    fsa addTransitionFrom: a to: b on: $a.
    fsa addTransitionFrom: b to: c on: $a.	
    fsa addTransitionFrom: c to: b on: $a.	

    fsa name: #nextTokenAAstarA.
    b priority: -1.
    c priority: 0.
    b retval: #AAstarA.

    scanner := (codeGenerator generate: fsa).

    self parse: 'a' token: #AAstarA rule: #nextTokenAAstarA.
    self parse: 'aaa' token: #AAstarA rule: #nextTokenAAstarA.
    self parse: 'aaaaa' token: #AAstarA rule: #nextTokenAAstarA.

    self fail: '' rule: #nextTokenAAstarA.
    self fail: 'aa' rule: #nextTokenAAstarA.
    self fail: 'aaaa' rule: #nextTokenAAstarA.
!

testAB
    fsa addState: a.
    fsa addState: b.
    fsa addState: c.
    fsa startState: a.
    fsa finalState: c.
    
    fsa addTransitionFrom: a to: b on: $a.
    fsa addTransitionFrom: b to: c on: $b.	
    
    fsa name: #nextTokenAB.
    c retval: #ab.
    
    scanner := (codeGenerator generate: fsa).
    
    self parse: 'ab' token: #ab rule: #nextTokenAB position: 2.
!

testABorBC
    fsa addState: a.
    fsa addState: b.
    fsa addState: c.
    fsa addState: d.
    fsa addState: e.
    fsa startState: a.
    fsa finalState: c.
    fsa finalState: e.
    
        
    fsa addTransitionFrom: a to: b on: $a.
    fsa addTransitionFrom: b to: c on: $b.	

    fsa addTransitionFrom: a to: d on: $b.
    fsa addTransitionFrom: d to: e on: $c.	

    fsa name: #nextTokenABorBC.
    c retval: #ab.
    e retval: #bc.

    scanner := (codeGenerator generate: fsa).

    self parse: 'ab' token: #ab rule: #nextTokenABorBC position: 2.
    self parse: 'abbc' token: #ab rule: #nextTokenABorBC position: 2.
    self parse: 'bc' token: #bc rule: #nextTokenABorBC position: 2.
    
    self fail: 'ac' rule: #nextTokenABorBC.
!

testABstarA
    fsa addState: a.
    fsa addState: b.
    fsa addState: c.
    fsa startState: a.
    fsa finalState: b.
    
    fsa addTransitionFrom: a to: b on: $a.
    fsa addTransitionFrom: b to: c on: $b.	
    fsa addTransitionFrom: c to: b on: $a.	

    fsa name: #nextTokenABstarA.
    b retval: #ABstarA.

    scanner := (codeGenerator generate: fsa).

    self parse: 'a' token: #ABstarA rule: #nextTokenABstarA position: 1.
    self parse: 'aa' token: #ABstarA rule: #nextTokenABstarA position: 1.
    self parse: 'aba' token: #ABstarA rule: #nextTokenABstarA position: 3.
    self parse: 'abaa' token: #ABstarA rule: #nextTokenABstarA position: 3.
    self parse: 'ababa' token: #ABstarA rule: #nextTokenABstarA position: 5.



    self fail: '' rule: #nextTokenABstarA.
!

testA_Bstar_A
    fsa addState: a.
    fsa addState: b.
    fsa addState: c.
    fsa startState: a.
    fsa finalState: c.
    
    fsa addTransitionFrom: a to: b on: $a.
    fsa addTransitionFrom: b to: b on: $b.	
    fsa addTransitionFrom: b to: c on: $a.	

    fsa name: #nextTokenA_Bstar_A.
    c retval: #A_Bstar_A.

    scanner := (codeGenerator generate: fsa).

    self parse: 'aa' token: #A_Bstar_A rule: #nextTokenA_Bstar_A.
    self parse: 'aba' token: #A_Bstar_A rule: #nextTokenA_Bstar_A.

    self fail: '' rule: #nextTokenA_Bstar_A.
!

testAorB
    fsa addState: a.
    fsa addState: b.
    fsa addState: c.
    fsa startState: a.
    fsa finalState: b.
    fsa finalState: c.
    
        
    fsa addTransitionFrom: a to: b on: $a.
    fsa addTransitionFrom: a to: c on: $b.	

    fsa name: #nextTokenAorB.
    b retval: #a.
    c retval: #b.

    scanner := (codeGenerator generate: fsa).

    self parse: 'a' token: #a rule: #nextTokenAorB.
    self parse: 'b' token: #b rule: #nextTokenAorB.	

    self fail: 'c' rule: #nextTokenAorB.	
    self fail: 'c' rule: #nextTokenAorB.		
!

testAstarA
    fsa addState: a.
    fsa addState: b.
    fsa startState: a.
    fsa finalState: b.
    
    fsa addTransitionFrom: a to: a on: $a.
    fsa addTransitionFrom: a to: b on: $a.	

    fsa name: #nextTokenAstarA.
    b retval: #AstarA.

    self should: [codeGenerator generate: fsa ] raise: Exception.
!

testAstarB
    fsa addState: a.
    fsa addState: b.
    fsa startState: a.
    fsa finalState: b.
    
    fsa addTransitionFrom: a to: a on: $a.
    fsa addTransitionFrom: a to: b on: $b.	

    fsa name: #nextTokenAstarB.
    b retval: #AstarB.

    scanner := (codeGenerator generate: fsa).
    
    self parse: 'ab' token: #AstarB rule: #nextTokenAstarB.
    self parse: 'b' token: #AstarB rule: #nextTokenAstarB.
    self parse: 'aaab' token: #AstarB rule: #nextTokenAstarB.

    self fail: 'c' rule: #nextTokenAstarB.	
! !

