"{ Package: 'stx:goodies/petitparser/compiler/tests/extras' }"

"{ NameSpace: Smalltalk }"

LibraryDefinition subclass:#stx_goodies_petitparser_compiler_tests_extras
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'* Projects & Packages *'
!


!stx_goodies_petitparser_compiler_tests_extras class methodsFor:'accessing'!

additionalClassAttributesFor: aClass
    "Answers additional set of class attributes for given class
     Individual project definitions may override this method, but
     overriding method should always merge its attributes with result
     of 'super additionalClassAttributesFor: aClass'.

     Here, we add #autoload attributes to all test cases and
     test resources, as they are not neccessary for the package
     and should not be compiled (because of unwanted dependency
     on stx:goodies/sunit package)

     But not make them autoloaded when the package is separate
     test-package - by conventions such package should by named
     #'module:package/subpackage/tests'
    "
    (TestCase notNil and:[aClass inheritsFrom: TestCase]) ifTrue:[^#()].
    (TestResource notNil and:[aClass inheritsFrom: TestResource]) ifTrue:[^#()].


    ^ super additionalClassAttributesFor: aClass

    "Created: / 10-05-2015 / 14:17:51 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!stx_goodies_petitparser_compiler_tests_extras class methodsFor:'accessing - monticello'!

monticelloLastMergedVersionInfo
    "The last merged version is: "

    ^ '
    Name: PetitCompiler-Extras-Tests-JanKurs.20150510144201
    Author: JanKurs
    Time: 10-05-2015, 04:59:25.308 PM
    UUID: ef43bd1a-be60-4e88-b749-8b635622c969
    Repository: http://smalltalkhub.com/mc/JanKurs/PetitParser/main

    '

    "Created: / 03-10-2014 / 02:27:21 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 12-05-2015 / 01:20:28 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

monticelloName
    "Return name of the package for Monticello. This is used when package is exported"

    ^ 'PetitCompiler-Extras-Tests'

    "Created: / 03-10-2014 / 01:47:26 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 10-05-2015 / 06:27:54 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!stx_goodies_petitparser_compiler_tests_extras class methodsFor:'description'!

excludedFromPreRequisites
    "list packages which are to be explicitely excluded from the automatic constructed
     prerequisites list. If empty, everything that is found along the inheritance of any of
     my classes is considered to be a prerequisite package."

    ^ #(
    )
!

mandatoryPreRequisites
    "list packages which are mandatory as a prerequisite.
     This are packages containing superclasses of my classes and classes which
     are extended by myself.
     They are mandatory, because we need these packages as a prerequisite for loading and compiling.
     This method is generated automatically,
     by searching along the inheritance chain of all of my classes."

    ^ #(
        #'stx:goodies/petitparser'    "PPCompositeParser - superclass of PPExpressionGrammar"
        #'stx:goodies/petitparser/parsers/java'    "PPJavaLexiconTest - superclass of PPCompiledJavaSyntaxTest"
        #'stx:goodies/petitparser/tests'    "PPAbstractParserTest - superclass of PPCCompiledJavaVerificationTest"
        #'stx:goodies/sunit'    "TestAsserter - superclass of PPCCompiledJavaVerificationTest"
        #'stx:libbasic'    "LibraryDefinition - superclass of stx_goodies_petitparser_compiler_tests_extras"
    )
!

referencedPreRequisites
    "list packages which are a prerequisite, because they contain
     classes which are referenced by my classes.
     We do not need these packages as a prerequisite for compiling or loading,
     however, a class from it may be referenced during execution and having it
     unloaded then may lead to a runtime doesNotUnderstand error, unless the caller
     includes explicit checks for the package being present.
     This method is generated automatically,
     by searching all classes (and their packages) which are referenced by my classes."

    ^ #(
        #'stx:goodies/petitparser/compiler'    "PPCArguments - referenced by PPCSmalltalkTests>>setUp"
        #'stx:goodies/petitparser/parsers/smalltalk'    "PPSmalltalkGrammar - referenced by PPCSmalltalkVerificationTest>>smalltalkGrammar"
        #'stx:libbasic2'    "Random - referenced by PPCResources>>expressionOfSize:stream:"
    )
!

subProjects
    "list packages which are known as subprojects.
     The generated makefile will enter those and make there as well.
     However: they are not forced to be loaded when a package is loaded;
     for those, redefine requiredPrerequisites."

    ^ #(
    )
! !

!stx_goodies_petitparser_compiler_tests_extras class methodsFor:'description - contents'!

classNamesAndAttributes
    "lists the classes which are to be included in the project.
     Each entry in the list may be: a single class-name (symbol),
     or an array-literal consisting of class name and attributes.
     Attributes are: #autoload or #<os> where os is one of win32, unix,..."

    ^ #(
        "<className> or (<className> attributes...) in load order"
        PPCCompiledJavaVerificationTest
        PPCExpressionsVerificationTest
        PPCResources
        PPCSmalltalkTests
        PPCSmalltalkVerificationTest
        PPCompiledExpressionGrammarResource
        PPCompiledExpressionGrammarTest
        PPCompiledJavaResource
        (PPCompiledJavaSyntaxTest autoload)
        PPCompiledSmalltalkGrammarResource
        PPCompiledSmalltalkGrammarTests
        (PPCompiledSmalltalkParserResource autoload)
        (PPCompiledSmalltalkParserTests autoload)
        PPExpressionGrammar
        PPExpressionGrammarTest
        PPLL1ExpressionGrammar
        PPLL1ExpressionGrammarTest
        PPTokenizedExpressionGrammarResource
        PPTokenizedExpressionGrammarTest
        PPTokenizedLL1ExpressionGrammarResource
        PPTokenizedLL1ExpressionGrammarTest
        PPTokenizedSmalltalkGrammarResource
        PPTokenizedSmalltalkGrammarTests
        (PPTokenizedSmalltalkParserResource autoload)
        (PPTokenizedSmalltalkParserTests autoload)
        #'stx_goodies_petitparser_compiler_tests_extras'
        PPCompiledExpressionsVerificationTest
        PPCompiledSmalltalkVerificationTest
        PPTokenizedExpressionsVerificationTest
        (PPTokenizedSmalltalkParserVerificationTest autoload)
        PPTokenizedSmalltalkVerificationTest
    )
!

extensionMethodNames
    "list class/selector pairs of extensions.
     A correponding method with real names must be present in my concrete subclasses"

    ^ #(
    )
! !

!stx_goodies_petitparser_compiler_tests_extras class methodsFor:'description - project information'!

companyName
    "Returns a company string which will appear in <lib>.rc.
     Under win32, this is placed into the dlls file-info"

    ^ 'eXept Software AG'
!

description
    "Returns a description string which will appear in nt.def / bc.def"

    ^ 'Smalltalk/X Class library'
!

legalCopyright
    "Returns a copyright string which will appear in <lib>.rc.
     Under win32, this is placed into the dlls file-info"

    ^ 'Copyright Claus Gittinger 1988-2015\nCopyright eXept Software AG 1998-2015'
!

productName
    "Returns a product name which will appear in <lib>.rc.
     Under win32, this is placed into the dlls file-info.
     This method is usually redefined in a concrete application definition"

    ^ 'Smalltalk/X'
! !

!stx_goodies_petitparser_compiler_tests_extras class methodsFor:'documentation'!

version_HG
    ^ '$Changeset: <not expanded> $'
! !

